from web3 import Web3
import hashlib

class Scan:
    infura_url = None 
    

    def infura_url_setter(self):
        url = str(input("Please enter your infura_url: "))
        if url == '/help':
            print("Please read the documentation under EthScan to view how to get an infura url")
        else:
            self.infura_url = url 

    def connection(self):
        self.web3 = Web3(Web3.HTTPProvider(self.infura_url))
        status = self.web3.isConnected()
        if status == True:
            return True 
        else:
            return False
    
    def get_latest_block(self):
        latest = self.web3.eth.blockNumber
        latest_block = self.web3.eth.getBlock(latest)
        return latest_block 

    def get_multiple_blocks(self, number):
        blocks = []
        for block in range(0, number):
            current_block = self.web3.eth.getBlock('latest')
            blocks.append(current_block)
    
        return blocks 

    def view_transaction_details(self, block_hash, trans_number):
        transaction_details = self.web3.eth.getTransactionByBlock(block_hash, trans_number)
        return transaction_details

    def mining(self, block_num, from_account_hash, transaction_hash, previous_hash, difficulty):
        nonce_limit = self.web3.eth.getTransactionCount(from_account_hash)
        for nonce in range(nonce_limit):
            base_text = str(block_num) + str(transaction_hash) + str(previous_hash) + str(nonce)
            hash_try = haslib.sha256(base_text.encode()).hexdigest()
            string = '0' * difficulty
            if hash_try[:difficulty] == string:
                print("Found hash with nonce: {}".format(nonce))
                return hash_try 

        return None        


ether_base = Scan()


import hashlib
import random 

nonce_limit = 1000000000

zeroes = random.randint(1, 100)

def mine(block_num, transaction_hash, previous_hash):
    for nonce in range(nonce_limit):
        base_text = str(block_num) + transaction_hash + previous_hash + str(nonce)
        hash_try = hashlib.sha256(base_text.encode()).hexdigest()
        if hash_try.startswith('0' * zeroes):
            print("Found hash with nonce: {}".format(nonce))
            return hash_try
    
    return None 





block_number = 24 
transactions = '761234fcc142'
previous_hash = "876de8756b967c87"

print(mine(block_number, transactions, previous_hash))



from ether_scan import ether_base
import time 


class EtherMine:
    blocks = []

    global_connection = None
    
    def connect_chain(self):
        ether_base.infura_url_setter()
        connection_status = ether_base.connection()
        if connection_status == False:
            self.global_connection = False
            return False 
        else:
            self.global_connection = True
            return True 

    def get_top_block(self):
        if not self.global_connection == True:
            return "Error - No Connection to Chain"
        latest_block = ether_base.get_latest_block()
        self.blocks.append(latest_block)
        return latest_block 

    def get_multiple_blocks(self):
        if not self.global_connection == True:
            return "Error - No Connection to Chain"
        num = int(input('Please type in the number of latest blocks you would like to see: '))
        blocks = ether_base.get_multiple_blocks(num)

        for block in blocks:
            self.blocks.append(block)

    def view_blocks(self):
        if not self.global_connection == True:
            return "Error - No Connection to Chain"
        return self.blocks

    def mine(self, block_num, from_account_hash, transaction_hash, previous_hash, difficulty):
        nonce_limit = self.web3.eth.getTransactionCount(from_account_hash)
        for nonce in range(nonce_limit):
            base_text = str(block_num) + str(transaction_hash) + str(previous_hash) + str(nonce)
            hash_try = haslib.sha256(base_text.encode()).hexdigest()
            if hash_try.startswith('0' * difficulty):
                print('Found hash with nonce: {}'.format(nonce))
                return hash_try

        return None     

                    
                    




                
        
        



from setuptools import setup 

setup(
    name = 'ethminer',
    version = '0.0.1',
    description= 'Ethereum Miner for Python',
    py_modules = ['ether_scan', 'scripting', 'beta', 'web3', 'hashlib'],
    package_dir = {'': 'src'},
)